@@ -13,7 +13,7 @@ extern "C" {
 
 
 /** Defines the Baresip version string */
-#define BARESIP_VERSION "0.4.12"
+#define BARESIP_VERSION "0.4.14"
 
 
 /* forward declarations */
@@ -56,6 +56,7 @@ enum call_event {
 	CALL_EVENT_ESTABLISHED,
 	CALL_EVENT_CLOSED,
 	CALL_EVENT_TRANSFER,
+	CALL_EVENT_TRANSFER_FAILED,
 };
 
 struct call;
@@ -214,9 +215,13 @@ enum presence_status {
 
 struct contact;
 
+int  contact_init(void);
+void contact_close(void);
 int  contact_add(struct contact **contactp, const struct pl *addr);
 int  contacts_print(struct re_printf *pf, void *unused);
 void contact_set_presence(struct contact *c, enum presence_status status);
+bool contact_block_access(const char *uri);
+struct contact  *contact_find(const char *uri);
 struct sip_addr *contact_addr(const struct contact *c);
 struct list     *contact_list(void);
 const char      *contact_str(const struct contact *c);
@@ -262,7 +267,7 @@ struct ausrc_prm {
 typedef void (ausrc_read_h)(const int16_t *sampv, size_t sampc, void *arg);
 typedef void (ausrc_error_h)(int err, const char *str, void *arg);
 
-typedef int  (ausrc_alloc_h)(struct ausrc_st **stp, struct ausrc *ausrc,
+typedef int  (ausrc_alloc_h)(struct ausrc_st **stp, const struct ausrc *ausrc,
 			     struct media_ctx **ctx,
 			     struct ausrc_prm *prm, const char *device,
 			     ausrc_read_h *rh, ausrc_error_h *errh, void *arg);
@@ -292,7 +297,7 @@ struct auplay_prm {
 
 typedef void (auplay_write_h)(int16_t *sampv, size_t sampc, void *arg);
 
-typedef int  (auplay_alloc_h)(struct auplay_st **stp, struct auplay *ap,
+typedef int  (auplay_alloc_h)(struct auplay_st **stp, const struct auplay *ap,
 			      struct auplay_prm *prm, const char *device,
 			      auplay_write_h *wh, void *arg);
 
@@ -463,6 +468,7 @@ enum ua_event {
 	UA_EVENT_CALL_PROGRESS,
 	UA_EVENT_CALL_ESTABLISHED,
 	UA_EVENT_CALL_CLOSED,
+	UA_EVENT_CALL_TRANSFER_FAILED,
 
 	UA_EVENT_MAX,
 };
@@ -503,6 +509,7 @@ const char     *ua_local_cuser(const struct ua *ua);
 struct account *ua_account(const struct ua *ua);
 const char     *ua_outbound(const struct ua *ua);
 struct call    *ua_call(const struct ua *ua);
+struct call    *ua_prev_call(const struct ua *ua);
 struct account *ua_prm(const struct ua *ua);
 struct list    *ua_calls(const struct ua *ua);
 enum presence_status ua_presence_status(const struct ua *ua);
@@ -606,7 +613,7 @@ struct vidsrc_prm {
 typedef void (vidsrc_frame_h)(struct vidframe *frame, void *arg);
 typedef void (vidsrc_error_h)(int err, void *arg);
 
-typedef int  (vidsrc_alloc_h)(struct vidsrc_st **vsp, struct vidsrc *vs,
+typedef int  (vidsrc_alloc_h)(struct vidsrc_st **vsp, const struct vidsrc *vs,
 			      struct media_ctx **ctx, struct vidsrc_prm *prm,
 			      const struct vidsz *size,
 			      const char *fmt, const char *dev,
@@ -641,7 +648,7 @@ struct vidisp_prm {
 typedef void (vidisp_resize_h)(const struct vidsz *size, void *arg);
 
 typedef int  (vidisp_alloc_h)(struct vidisp_st **vp,
-			      struct vidisp *vd, struct vidisp_prm *prm,
+			      const struct vidisp *vd, struct vidisp_prm *prm,
 			      const char *dev,
 			      vidisp_resize_h *resizeh, void *arg);
 typedef int  (vidisp_update_h)(struct vidisp_st *st, bool fullscreen,
@@ -731,10 +738,10 @@ typedef int (videnc_packet_h)(bool marker, const uint8_t *hdr, size_t hdr_len,
 
 typedef int (videnc_update_h)(struct videnc_state **vesp,
 			      const struct vidcodec *vc,
-			      struct videnc_param *prm, const char *fmtp);
-typedef int (videnc_encode_h)(struct videnc_state *ves, bool update,
-			      const struct vidframe *frame,
+			      struct videnc_param *prm, const char *fmtp,
 			      videnc_packet_h *pkth, void *arg);
+typedef int (videnc_encode_h)(struct videnc_state *ves, bool update,
+			      const struct vidframe *frame);
 
 typedef int (viddec_update_h)(struct viddec_state **vdsp,
 			      const struct vidcodec *vc, const char *fmtp);
@@ -758,6 +765,8 @@ struct vidcodec {
 void vidcodec_register(struct vidcodec *vc);
 void vidcodec_unregister(struct vidcodec *vc);
 const struct vidcodec *vidcodec_find(const char *name, const char *variant);
+const struct vidcodec *vidcodec_find_encoder(const char *name);
+const struct vidcodec *vidcodec_find_decoder(const char *name);
 struct list *vidcodec_list(void);
 
 
@@ -894,6 +903,9 @@ const char *sdp_rattr(const struct sdp_session *s, const struct sdp_media *m,
 #endif
 
 
+int module_preload(const char *module);
+
+
 #ifdef __cplusplus
 }
 #endif
