@@ -26,13 +26,12 @@ extern const uint8_t h264_level_idc;
 struct videnc_state;
 
 int encode_update(struct videnc_state **vesp, const struct vidcodec *vc,
-		  struct videnc_param *prm, const char *fmtp);
-int encode(struct videnc_state *st, bool update, const struct vidframe *frame,
-	   videnc_packet_h *pkth, void *arg);
+		  struct videnc_param *prm, const char *fmtp,
+		  videnc_packet_h *pkth, void *arg);
+int encode(struct videnc_state *st, bool update, const struct vidframe *frame);
 #ifdef USE_X264
 int encode_x264(struct videnc_state *st, bool update,
-		const struct vidframe *frame,
-		videnc_packet_h *pkth, void *arg);
+		const struct vidframe *frame);
 #endif
 
 
