@@ -8,6 +8,16 @@
 #include <baresip.h>
 
 
+/**
+ * @defgroup menu menu
+ *
+ * Interactive menu
+ *
+ * This module must be loaded if you want to use the interactive menu
+ * to control the Baresip application.
+ */
+
+
 /** Defines the status modes */
 enum statmode {
 	STATMODE_CALL = 0,
@@ -302,6 +312,58 @@ static int cmd_hangup(struct re_printf *pf, void *unused)
 }
 
 
+static int create_ua(struct re_printf *pf, void *arg)
+{
+	const struct cmd_arg *carg = arg;
+	struct le *le;
+	int err = 0;
+
+	(void)pf;
+
+	 if (str_isset(carg->prm)) {
+
+		mbuf_rewind(dialbuf);
+		(void)mbuf_write_str(dialbuf, carg->prm);
+
+		(void)re_hprintf(pf, "Creating UA for %s ...\n", carg->prm);
+		err = ua_alloc(NULL, carg->prm);
+
+
+	}
+	else if (dialbuf->end > 0) {
+
+		char *uri;
+
+		dialbuf->pos = 0;
+		err = mbuf_strdup(dialbuf, &uri, dialbuf->end);
+		if (err)
+			return err;
+
+		(void)re_hprintf(pf, "Creating UA for %s ...\n", uri);
+		err |=  ua_alloc(NULL, uri);
+
+		mem_deref(uri);
+	}
+
+	for (le = list_head(uag_list()); le && !err; le = le->next) {
+		const struct ua *ua = le->data;
+
+		err  = re_hprintf(pf, "%s ", ua == uag_cur() ? ">" : " ");
+		err |= ua_print_status(pf, ua);
+	}
+
+	err |= re_hprintf(pf, "\n");
+
+
+	if (err) {
+		(void)re_hprintf(pf, "menu: create_ua failed: %m\n", err);
+	}
+
+
+	return err;
+}
+
+
 static int cmd_ua_next(struct re_printf *pf, void *unused)
 {
 	(void)pf;
@@ -309,6 +371,8 @@ static int cmd_ua_next(struct re_printf *pf, void *unused)
 
 	if (!le_cur)
 		le_cur = list_head(uag_list());
+	if (!le_cur)
+		return 0;
 
 	le_cur = le_cur->next ? le_cur->next : list_head(uag_list());
 
@@ -363,8 +427,9 @@ static const struct cmd cmdv[] = {
 	{'y',       0, "Memory status",            mem_status           },
 	{0x1b,      0, "Hangup call",              cmd_hangup           },
 	{' ',       0, "Toggle UAs",               cmd_ua_next          },
+	{'T',       0, "Toggle UAs",               cmd_ua_next          },
 	{'g',       0, "Print configuration",      cmd_config_print     },
-
+	{'R', CMD_PRM, "Create User-Agent",        create_ua            },
 	{'#', CMD_PRM, NULL,   dial_handler },
 	{'*', CMD_PRM, NULL,   dial_handler },
 	{'0', CMD_PRM, NULL,   dial_handler },
@@ -448,6 +513,14 @@ static int call_holdresume(struct re_printf *pf, void *arg)
 }
 
 
+static int hold_prev_call(struct re_printf *pf, void *unused)
+{
+	(void)pf;
+	(void)unused;
+	return call_hold(ua_prev_call(uag_cur()), true);
+}
+
+
 #ifdef USE_VIDEO
 static int call_videoenc_cycle(struct re_printf *pf, void *unused)
 {
@@ -504,6 +577,7 @@ static const struct cmd callcmdv[] = {
 	{'m',       0, "Call mute/un-mute",   call_mute             },
 	{'r', CMD_IPRM,"Transfer call",       call_xfer             },
 	{'x',       0, "Call hold",           call_holdresume       },
+	{'H',       0, "Hold previous call",  hold_prev_call        },
 
 #ifdef USE_VIDEO
 	{'E',       0, "Cycle video encoder", call_videoenc_cycle   },
