@@ -18,7 +18,7 @@
 #include <re.h>
 #include <rem.h>
 #include <baresip.h>
-#if defined (OPENBSD)
+#if defined (OPENBSD) || defined (NETBSD)
 #include <sys/videoio.h>
 #else
 #include <linux/videodev2.h>
@@ -42,7 +42,7 @@ struct buffer {
 };
 
 struct vidsrc_st {
-	struct vidsrc *vs;  /* inheritance */
+	const struct vidsrc *vs;  /* inheritance */
 
 	int fd;
 	pthread_t thread;
@@ -402,8 +402,6 @@ static void destructor(void *arg)
 
 	if (st->fd >= 0)
 		v4l2_close(st->fd);
-
-	mem_deref(st->vs);
 }
 
 
@@ -423,7 +421,7 @@ static void *read_thread(void *arg)
 }
 
 
-static int alloc(struct vidsrc_st **stp, struct vidsrc *vs,
+static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 		 struct media_ctx **ctx, struct vidsrc_prm *prm,
 		 const struct vidsz *size, const char *fmt,
 		 const char *dev, vidsrc_frame_h *frameh,
@@ -447,7 +445,7 @@ static int alloc(struct vidsrc_st **stp, struct vidsrc *vs,
 	if (!st)
 		return ENOMEM;
 
-	st->vs = mem_ref(vs);
+	st->vs = vs;
 	st->fd = -1;
 	st->sz = *size;
 	st->frameh = frameh;
