@@ -11,6 +11,12 @@
 #include <baresip.h>
 
 
+/** Internal pixel-format */
+#ifndef VIDLOOP_INTERNAL_FMT
+#define VIDLOOP_INTERNAL_FMT (VID_FMT_YUV420P)
+#endif
+
+
 /** Video Statistics */
 struct vstat {
 	uint64_t tsamp;
@@ -23,7 +29,8 @@ struct vstat {
 
 /** Video loop */
 struct video_loop {
-	const struct vidcodec *vc;
+	const struct vidcodec *vc_enc;
+	const struct vidcodec *vc_dec;
 	struct config_video cfg;
 	struct videnc_state *enc;
 	struct viddec_state *dec;
@@ -34,6 +41,8 @@ struct video_loop {
 	struct vstat stat;
 	struct tmr tmr_bw;
 	uint16_t seq;
+	bool need_conv;
+	int err;
 };
 
 
@@ -57,8 +66,17 @@ static int display(struct video_loop *vl, struct vidframe *frame)
 			err |= st->vf->dech(st, frame);
 	}
 
+	if (err) {
+		warning("vidloop: error in video-filters (%m)\n", err);
+	}
+
 	/* display frame */
-	(void)vidisp_display(vl->vidisp, "Video Loop", frame);
+	err = vidisp_display(vl->vidisp, "Video Loop", frame);
+	if (err == ENODEV) {
+		info("vidloop: video-display was closed\n");
+		vl->vidisp = mem_deref(vl->vidisp);
+		vl->err = err;
+	}
 
 	return err;
 }
@@ -86,8 +104,8 @@ static int packet_handler(bool marker, const uint8_t *hdr, size_t hdr_len,
 
 	/* decode */
 	frame.data[0] = NULL;
-	if (vl->dec) {
-		err = vl->vc->dech(vl->dec, &frame, marker, vl->seq++, mb);
+	if (vl->vc_dec && vl->dec) {
+		err = vl->vc_dec->dech(vl->dec, &frame, marker, vl->seq++, mb);
 		if (err) {
 			warning("vidloop: codec decode: %m\n", err);
 			goto out;
@@ -112,9 +130,17 @@ static void vidsrc_frame_handler(struct vidframe *frame, void *arg)
 
 	++vl->stat.frames;
 
-	if (frame->fmt != VID_FMT_YUV420P) {
+	if (frame->fmt != VIDLOOP_INTERNAL_FMT) {
 
-		if (vidframe_alloc(&f2, VID_FMT_YUV420P, &frame->size))
+		if (!vl->need_conv) {
+			info("vidloop: NOTE: pixel-format conversion"
+			     " needed: %s  -->  %s\n",
+			     vidfmt_name(frame->fmt),
+			     vidfmt_name(VIDLOOP_INTERNAL_FMT));
+			vl->need_conv = true;
+		}
+
+		if (vidframe_alloc(&f2, VIDLOOP_INTERNAL_FMT, &frame->size))
 			return;
 
 		vidconv(f2, frame, 0);
@@ -131,9 +157,8 @@ static void vidsrc_frame_handler(struct vidframe *frame, void *arg)
 			err |= st->vf->ench(st, frame);
 	}
 
-	if (vl->enc) {
-		(void)vl->vc->ench(vl->enc, false, frame,
-				   packet_handler, vl);
+	if (vl->vc_enc && vl->enc) {
+		(void)vl->vc_enc->ench(vl->enc, false, frame);
 	}
 	else {
 		vl->stat.bytes += vidframe_size(frame->fmt, &frame->size);
@@ -150,9 +175,9 @@ static void vidloop_destructor(void *arg)
 
 	tmr_cancel(&vl->tmr_bw);
 	mem_deref(vl->vsrc);
-	mem_deref(vl->vidisp);
 	mem_deref(vl->enc);
 	mem_deref(vl->dec);
+	mem_deref(vl->vidisp);
 	list_flush(&vl->filtencl);
 	list_flush(&vl->filtdecl);
 }
@@ -161,26 +186,42 @@ static void vidloop_destructor(void *arg)
 static int enable_codec(struct video_loop *vl)
 {
 	struct videnc_param prm;
+	const char *name = NULL;
 	int err;
 
 	prm.fps     = vl->cfg.fps;
-	prm.pktsize = 1024;
+	prm.pktsize = 1480;
 	prm.bitrate = vl->cfg.bitrate;
 	prm.max_fs  = -1;
 
 	/* Use the first video codec */
-	vl->vc = vidcodec_find(NULL, NULL);
-	if (!vl->vc)
-		return ENOENT;
 
-	err = vl->vc->encupdh(&vl->enc, vl->vc, &prm, NULL);
+	vl->vc_enc = vidcodec_find_encoder(name);
+	if (!vl->vc_enc) {
+		warning("vidloop: could not find encoder (%s)\n", name);
+		return ENOENT;
+	}
+
+	info("vidloop: enabled encoder %s (%u fps, %u bit/s)\n",
+	     vl->vc_enc->name, prm.fps, prm.bitrate);
+
+	vl->vc_dec = vidcodec_find_decoder(name);
+	if (!vl->vc_dec) {
+		warning("vidloop: could not find decoder (%s)\n", name);
+		return ENOENT;
+	}
+
+	info("vidloop: enabled decoder %s\n", vl->vc_dec->name);
+
+	err = vl->vc_enc->encupdh(&vl->enc, vl->vc_enc, &prm, NULL,
+				  packet_handler, vl);
 	if (err) {
 		warning("vidloop: update encoder failed: %m\n", err);
 		return err;
 	}
 
-	if (vl->vc->decupdh) {
-		err = vl->vc->decupdh(&vl->dec, vl->vc, NULL);
+	if (vl->vc_dec->decupdh) {
+		err = vl->vc_dec->decupdh(&vl->dec, vl->vc_dec, NULL);
 		if (err) {
 			warning("vidloop: update decoder failed: %m\n", err);
 			return err;
@@ -195,13 +236,18 @@ static void disable_codec(struct video_loop *vl)
 {
 	vl->enc = mem_deref(vl->enc);
 	vl->dec = mem_deref(vl->dec);
-	vl->vc = NULL;
+	vl->vc_enc = NULL;
+	vl->vc_dec = NULL;
 }
 
 
 static void print_status(struct video_loop *vl)
 {
-	(void)re_fprintf(stderr, "\rstatus: EFPS=%.1f      %u kbit/s       \r",
+	(void)re_fprintf(stderr,
+			 "\rstatus:"
+			 " [%s] [%s] EFPS=%.1f      %u kbit/s       \r",
+			 vl->vc_enc ? vl->vc_enc->name : "",
+			 vl->vc_dec ? vl->vc_dec->name : "",
 			 vl->stat.efps, vl->stat.bitrate);
 }
 
@@ -229,7 +275,13 @@ static void timeout_bw(void *arg)
 {
 	struct video_loop *vl = arg;
 
-	tmr_start(&vl->tmr_bw, 5000, timeout_bw, vl);
+	if (vl->err) {
+		info("error in video-loop -- closing (%m)\n", vl->err);
+		gvl = mem_deref(gvl);
+		return;
+	}
+
+	tmr_start(&vl->tmr_bw, 2000, timeout_bw, vl);
 
 	calc_bitrate(vl);
 	print_status(vl);
@@ -334,14 +386,19 @@ static int vidloop_start(struct re_printf *pf, void *arg)
 	size.h = cfg->video.height;
 
 	if (gvl) {
-		if (gvl->vc)
+		if (gvl->vc_enc)
 			disable_codec(gvl);
-		else
-			(void)enable_codec(gvl);
+		else {
+			err = enable_codec(gvl);
+			if (err) {
+				gvl = mem_deref(gvl);
+				return err;
+			}
+		}
 
 		(void)re_hprintf(pf, "%sabled codec: %s\n",
-				 gvl->vc ? "En" : "Dis",
-				 gvl->vc ? gvl->vc->name : "");
+				 gvl->vc_enc ? "En" : "Dis",
+				 gvl->vc_enc ? gvl->vc_enc->name : "");
 	}
 	else {
 		(void)re_hprintf(pf, "Enable video-loop on %s,%s: %u x %u\n",
