@@ -22,7 +22,7 @@
 
 
 struct vidsrc_st {
-	struct vidsrc *vs;  /* inheritance */
+	const struct vidsrc *vs;  /* inheritance */
 	Display *disp;
 	XImage *image;
 	pthread_t thread;
@@ -144,12 +144,10 @@ static void destructor(void *arg)
 
 	if (st->disp)
 		XCloseDisplay(st->disp);
-
-	mem_deref(st->vs);
 }
 
 
-static int alloc(struct vidsrc_st **stp, struct vidsrc *vs,
+static int alloc(struct vidsrc_st **stp, const struct vidsrc *vs,
 		 struct media_ctx **ctx, struct vidsrc_prm *prm,
 		 const struct vidsz *size, const char *fmt,
 		 const char *dev, vidsrc_frame_h *frameh,
@@ -170,7 +168,7 @@ static int alloc(struct vidsrc_st **stp, struct vidsrc *vs,
 	if (!st)
 		return ENOMEM;
 
-	st->vs     = mem_ref(vs);
+	st->vs     = vs;
 	st->size   = *size;
 	st->fps    = prm->fps;
 	st->frameh = frameh;
