@@ -276,7 +276,7 @@ static int verify_sas(struct re_printf *pf, void *arg)
 		char *s2h;
 		char rzid[ZRTP_STRING16] = "";
 		zrtp_status_t s;
-		zrtp_string16_t remote_zid;
+		zrtp_string16_t remote_zid = ZSTR_INIT_EMPTY(remote_zid);
 
 		if (str_len(carg->prm) != 24) {
 			warning("zrtp: invalid remote ZID (%s)\n", carg->prm);
@@ -289,7 +289,8 @@ static int verify_sas(struct re_printf *pf, void *arg)
 			warning("zrtp: str2hex failed (%s)\n", s2h);
 			return EINVAL;
 		}
-		zrtp_zstrcpyc(ZSTR_GV(remote_zid), rzid);
+		zrtp_zstrncpyc(ZSTR_GV(remote_zid), (const char*)rzid,
+			       sizeof(zrtp_zid_t));
 
 		s = zrtp_cache_set_verified(zrtp_global->cache,
 					    ZSTR_GV(remote_zid),
