@@ -391,6 +391,18 @@ static void audio_error_handler(int err, const char *str, void *arg)
 }
 
 
+static void video_error_handler(int err, const char *str, void *arg)
+{
+	struct call *call = arg;
+	MAGIC_CHECK(call);
+
+	warning("call: video device error: %m (%s)\n", err, str);
+
+	call_stream_stop(call);
+	call_event_handler(call, CALL_EVENT_CLOSED, str);
+}
+
+
 static void menc_error_handler(int err, void *arg)
 {
 	struct call *call = arg;
@@ -523,7 +535,8 @@ int call_alloc(struct call **callp, const struct config *cfg, struct list *lst,
 				  acc->mnat, call->mnats,
 				  acc->menc, call->mencs,
 				  "main",
-				  account_vidcodecl(call->acc));
+				  account_vidcodecl(call->acc),
+				  video_error_handler, call);
 		if (err)
 			goto out;
  	}
@@ -1504,6 +1517,8 @@ static void sipsub_notify_handler(struct sip *sip, const struct sip_msg *msg,
 
 	if (sc >= 300) {
 		warning("call: transfer failed: %u %r\n", sc, &reason);
+		call_event_handler(call, CALL_EVENT_TRANSFER_FAILED,
+				   "%u %r", sc, &reason);
 	}
 	else if (sc >= 200) {
 		call_event_handler(call, CALL_EVENT_CLOSED, "Call transfered");
@@ -1527,6 +1542,8 @@ static void sipsub_close_handler(int err, const struct sip_msg *msg,
 	else if (msg && msg->scode >= 300) {
 		info("call: transfer failed: %u %r\n",
 		     msg->scode, &msg->reason);
+		call_event_handler(call, CALL_EVENT_TRANSFER_FAILED,
+				   "%u %r", msg->scode, &msg->reason);
 	}
 }
 
