@@ -379,8 +379,12 @@ static int core_config_template(struct re_printf *pf, const struct config *cfg)
 			  "\n# Core\n"
 			  "poll_method\t\t%s\t\t# poll, select"
 #ifdef HAVE_EPOLL
-				", epoll ..\n"
+				", epoll .."
 #endif
+#ifdef HAVE_KQUEUE
+				", kqueue .."
+#endif
+				"\n"
 			  "\n# SIP\n"
 			  "sip_trans_bsize\t\t128\n"
 			  "#sip_listen\t\t0.0.0.0:5060\n"
@@ -678,6 +682,9 @@ int config_write_template(const char *file, const struct config *cfg)
 	(void)re_fprintf(f, "\n");
 	(void)re_fprintf(f, "cons_listen\t\t0.0.0.0:5555\n");
 
+	(void)re_fprintf(f, "\n");
+	(void)re_fprintf(f, "http_listen\t\t0.0.0.0:8000\n");
+
 	(void)re_fprintf(f, "\n");
 	(void)re_fprintf(f, "evdev_device\t\t/dev/input/event0\n");
 
@@ -685,10 +692,15 @@ int config_write_template(const char *file, const struct config *cfg)
 	(void)re_fprintf(f, "speex_quality\t\t7 # 0-10\n");
 	(void)re_fprintf(f, "speex_complexity\t7 # 0-10\n");
 	(void)re_fprintf(f, "speex_enhancement\t0 # 0-1\n");
+	(void)re_fprintf(f, "speex_mode_nb\t\t3 # 1-6\n");
+	(void)re_fprintf(f, "speex_mode_wb\t\t6 # 1-6\n");
 	(void)re_fprintf(f, "speex_vbr\t\t0 # Variable Bit Rate 0-1\n");
 	(void)re_fprintf(f, "speex_vad\t\t0 # Voice Activity Detection 0-1\n");
 	(void)re_fprintf(f, "speex_agc_level\t\t8000\n");
 
+	(void)re_fprintf(f, "\n# Opus codec parameters\n");
+	(void)re_fprintf(f, "opus_bitrate\t\t28000 # 6000-510000\n");
+
 	(void)re_fprintf(f, "\n# NAT Behavior Discovery\n");
 	(void)re_fprintf(f, "natbd_server\t\tcreytiv.com\n");
 	(void)re_fprintf(f, "natbd_interval\t\t600\t\t# in seconds\n");
