@@ -7,6 +7,7 @@
 #include <re.h>
 #include <baresip.h>
 #include "core.h"
+#include <ctype.h>
 
 
 /** Magic number */
@@ -266,6 +267,16 @@ static void call_event_handler(struct call *call, enum call_event ev,
 	switch (ev) {
 
 	case CALL_EVENT_INCOMING:
+
+		if (contact_block_access(peeruri)) {
+
+			info("ua: blocked access: \"%s\"\n", peeruri);
+
+			ua_event(ua, UA_EVENT_CALL_CLOSED, call, str);
+			mem_deref(call);
+			break;
+		}
+
 		switch (ua->acc->answermode) {
 
 		case ANSWERMODE_EARLY:
@@ -349,6 +360,10 @@ static void call_event_handler(struct call *call, enum call_event ev,
 			mem_deref(call2);
 		}
 		break;
+
+	case CALL_EVENT_TRANSFER_FAILED:
+		ua_event(ua, UA_EVENT_CALL_TRANSFER_FAILED, call, str);
+		break;
 	}
 }
 
@@ -596,6 +611,10 @@ static int uri_complete(struct ua *ua, struct mbuf *buf, const char *uri)
 	size_t len;
 	int err = 0;
 
+	/* Skip initial whitespace */
+	while (isspace(*uri))
+		++uri;
+
 	len = str_len(uri);
 
 	/* Append sip: scheme if missing */
@@ -904,6 +923,27 @@ struct call *ua_call(const struct ua *ua)
 }
 
 
+struct call *ua_prev_call(const struct ua *ua)
+{
+	struct le *le;
+	int prev = 0;
+
+	if (!ua)
+		return NULL;
+
+	for (le = ua->calls.tail; le; le = le->prev) {
+		if ( prev == 1) {
+			struct call *call = le->data;
+			return call;
+		}
+		if ( prev == 0)
+			prev = 1;
+	}
+
+	return NULL;
+}
+
+
 int ua_debug(struct re_printf *pf, const struct ua *ua)
 {
 	struct le *le;
@@ -1157,6 +1197,10 @@ int ua_init(const char *software, bool udp, bool tcp, bool tls,
 
 	play_init();
 
+	err = contact_init();
+	if (err)
+		return err;
+
 	/* Initialise Network */
 	err = net_init(&cfg->net, prefer_ipv6 ? AF_INET6 : AF_INET);
 	if (err) {
@@ -1219,6 +1263,7 @@ void ua_close(void)
 	net_close();
 	play_close();
 	ui_reset();
+	contact_close();
 
 	uag.evsock   = mem_deref(uag.evsock);
 	uag.sock     = mem_deref(uag.sock);
@@ -1325,6 +1370,10 @@ int ua_print_calls(struct re_printf *pf, const struct ua *ua)
 {
 	struct le *le;
 	int err = 0;
+	if (!ua) {
+		err |= re_hprintf(pf, "\n--- No active calls ---\n");
+		return err;
+	}
 
 	err |= re_hprintf(pf, "\n--- List of active calls (%u): ---\n",
 			  list_count(&ua->calls));
@@ -1396,15 +1445,16 @@ const char *uag_event_str(enum ua_event ev)
 {
 	switch (ev) {
 
-	case UA_EVENT_REGISTERING:      return "REGISTERING";
-	case UA_EVENT_REGISTER_OK:      return "REGISTER_OK";
-	case UA_EVENT_REGISTER_FAIL:    return "REGISTER_FAIL";
-	case UA_EVENT_UNREGISTERING:    return "UNREGISTERING";
-	case UA_EVENT_CALL_INCOMING:    return "CALL_INCOMING";
-	case UA_EVENT_CALL_RINGING:     return "CALL_RINGING";
-	case UA_EVENT_CALL_PROGRESS:    return "CALL_PROGRESS";
-	case UA_EVENT_CALL_ESTABLISHED: return "CALL_ESTABLISHED";
-	case UA_EVENT_CALL_CLOSED:      return "CALL_CLOSED";
+	case UA_EVENT_REGISTERING:          return "REGISTERING";
+	case UA_EVENT_REGISTER_OK:          return "REGISTER_OK";
+	case UA_EVENT_REGISTER_FAIL:        return "REGISTER_FAIL";
+	case UA_EVENT_UNREGISTERING:        return "UNREGISTERING";
+	case UA_EVENT_CALL_INCOMING:        return "CALL_INCOMING";
+	case UA_EVENT_CALL_RINGING:         return "CALL_RINGING";
+	case UA_EVENT_CALL_PROGRESS:        return "CALL_PROGRESS";
+	case UA_EVENT_CALL_ESTABLISHED:     return "CALL_ESTABLISHED";
+	case UA_EVENT_CALL_CLOSED:          return "CALL_CLOSED";
+	case UA_EVENT_CALL_TRANSFER_FAILED: return "TRANSFER_FAILED";
 	default: return "?";
 	}
 }
